@@ -7,29 +7,31 @@ import re
 from rescape_graphene.graphql_helpers.schema_helpers import process_filter_kwargs
 
 
-def quiz_model_query(client, model_query_function, result_name, variables):
+def quiz_model_query(client, model_query_function, result_name, variables, expect_length=1):
     """
         Tests a query for a model with variables that produce exactly one result
     :param client: Apollo client
     :param model_query_function: Query function expecting the client and variables
     :param result_name: The name of the result object in the data object
     :param variables: key value variables for the query
-    :return: void
+    :param expect_length: Default 1. Optional number items to expect
+    :return: returns the result for further assertions
     """
     all_result = model_query_function(client)
-    assert not R.has('errors', all_result), R.dump_json(R.prop('errors', all_result))
+    assert not R.has('errors', all_result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', all_result))))
     result = model_query_function(
         client,
         variables=variables
     )
     # Check against errors
-    assert not R.has('errors', result), R.dump_json(R.prop('errors', result))
+    assert not R.has('errors', result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', result))))
     # Simple assertion that the query looks good
-    assert 1 == R.length(R.item_path(['data', result_name], result))
+    assert expect_length == R.length(R.item_path(['data', result_name], result))
+    return result
 
 
 def quiz_model_paginated_query(client, model_class, paginated_query, result_name, page_count_expected, props,
-                               omit_props):
+                               omit_props, order_by=None, page_size=1):
     """
         Tests a pagination query for a model with variables
     :param client: Apollo client
@@ -40,21 +42,26 @@ def quiz_model_paginated_query(client, model_class, paginated_query, result_name
     :param result_name: The name of the results in data.[result_name].objects
     :param props: The props to query, not including pagination
     :param omit_props: Props to omit from assertions because they are nondeterminate
-    :return:
+    :param order_by: Order by page-level prop
+    :param page_size: Default 1
+    :return the first result (first page) and final result (last page) for further testing:
     """
     result = paginated_query(
         client,
         variables=dict(
             page=1,
-            page_size=1,
+            page_size=page_size,
+            order_by=order_by,
             objects=R.to_array_if_not(props)
         )
     )
+
+
     # Check against errors
-    assert not R.has('errors', result), R.dump_json(R.prop('errors', result))
+    assert not R.has('errors', result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', result))))
     first_page_objects = R.item_path(['data', result_name, 'objects'], result)
     # Assert we got 1 result because our page is size 1
-    assert 1 == R.compose(
+    assert page_size == R.compose(
         R.length,
         R.map(R.omit(omit_props)),
     )(first_page_objects)
@@ -64,15 +71,16 @@ def quiz_model_paginated_query(client, model_class, paginated_query, result_name
             R.map(
                 R.prop('id'),
                 model_class.objects.filter(
-                    *process_filter_kwargs(model_class, R.map_keys(underscore, props))
-                )
+                    *process_filter_kwargs(model_class, **R.map_keys(underscore, props))
+                ).order_by(*order_by.split(','))
             )
         ) -
         set(R.map(R.compose(int, R.prop('id')), first_page_objects))
     )
 
     page_info = R.item_path(['data', result_name], result)
-    # We have 1 page pages and thus expect 2 pages that match Oslo
+    # We have page_size pages so there should be a total number of pages
+    # of what we specified for page_count_expected
     assert page_info['pages'] == page_count_expected
     assert page_info['hasNext'] == True
     assert page_info['hasPrev'] == False
@@ -82,6 +90,7 @@ def quiz_model_paginated_query(client, model_class, paginated_query, result_name
         variables=dict(
             page=page_count_expected,
             page_size=page_info['pageSize'],
+            order_by=order_by,
             objects=R.to_array_if_not(props)
         )
     )
@@ -97,6 +106,7 @@ def quiz_model_paginated_query(client, model_class, paginated_query, result_name
     # Make sure it's the last page
     assert new_page_info['hasNext'] == False
     assert new_page_info['hasPrev'] == True
+    return [result, new_result]
 
 
 def quiz_model_versioned_query(client, model_class, model_query, result_name, version_count_expected, props,
@@ -122,7 +132,7 @@ def quiz_model_versioned_query(client, model_class, model_query, result_name, ve
         )
     )
     # Check against errors
-    assert not R.has('errors', result), R.dump_json(R.prop('errors', result))
+    assert not R.has('errors', result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', result))))
     assert R.compose(
         R.length,
         R.item_str_path_or([], f'data.{result_name}.objects')
@@ -146,7 +156,7 @@ def quiz_model_mutation_create(client, graphql_update_or_create_function, result
     result = graphql_update_or_create_function(client, values=values)
 
     result_path_partial = R.item_str_path(f'data.{result_path}')
-    assert not R.has('errors', result), R.dump_json(R.prop('errors', result))
+    assert not R.has('errors', result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', result))))
     # Get the created value, using underscore to make the camelcase keys match python keys
     created = R.map_keys(
         lambda key: underscore(key),
@@ -157,7 +167,7 @@ def quiz_model_mutation_create(client, graphql_update_or_create_function, result
     # Try creating with the same values again, unique constraints will apply to force a create or an update will occur
     if second_create_results:
         new_result = graphql_update_or_create_function(client, values)
-        assert not R.has('errors', new_result), R.dump_json(R.prop('errors', new_result))
+        assert not R.has('errors', new_result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', new_result))))
         created_too = result_path_partial(new_result)
         if second_create_does_update:
             assert created['id'] == created_too['id']
@@ -185,7 +195,7 @@ def quiz_model_mutation_update(client, graphql_update_or_create_function, create
     :return:
     """
     result = graphql_update_or_create_function(client, values=values)
-    assert not R.has('errors', result), R.dump_json(R.prop('errors', result))
+    assert not R.has('errors', result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', result))))
     # Extract the result and map the graphql keys to match the python keys
     created = R.compose(
         lambda r: R.map_keys(lambda key: underscore(key), r),
@@ -206,7 +216,7 @@ def quiz_model_mutation_update(client, graphql_update_or_create_function, create
             update_values
         ])
     )
-    assert not R.has('errors', update_result), R.dump_json(R.prop('errors', update_result))
+    assert not R.has('errors', update_result), R.dump_json(R.map(lambda e: format_error(e), R.dump_json(R.prop('errors', update_result))))
     updated = R.item_str_path(f'data.{update_path}', update_result)
     assert created['id'] == updated['id']
     assert update_values == pick_deep(
